package Cliente.Transformacional;

import CWMTransformation.*;
import CWMTransformationImpl.*;
import Core.*;
import CoreImpl.*;
import Cliente.Util.PrintUtil;

import org.omg.CORBA.ORB;
import Cliente.Util.IorUtils;
import java.io.*;
import java.util.Properties;
import java.net.*;
import java.sql.Date;
import java.sql.Timestamp;
import Servidor.*;

public class ClienteCorba {

  private static org.omg.CORBA.ORB orb;
  private static String REPOS_IOR_FILE="ClienteCorba83720.41133134613";


  static public void main(String[] args) {
    _CWMTransformationPackage pkg;
    _CorePackage pkgCore;

    String accion = getAccion(args);
    if (accion==null) return;



    try {
      IorUtils iorUtils = new IorUtils(orb);

      if (accion.equals("--list")){
        pkg = (_CWMTransformationPackage)iorUtils.loadExistingRefPkg(
          "_CWMTransformationPackage",REPOS_IOR_FILE);
        PrintUtil.PrintPackageTransformacional(pkg);
      }
      else if (accion.equals("--addnew")){
        pkgCore  = (_CorePackage)iorUtils.loadNewRefPkg("_CorePackage");
        pkg = (_CWMTransformationPackage)iorUtils.loadExistingRefPkg(
          "_CWMTransformationPackage",REPOS_IOR_FILE);
        crearTransformation(pkg,pkgCore);
        PrintUtil.PrintPackageTransformacional(pkg);
      }
      else{

        PrintUtil.Debug("\nObteniendo nombre de la referencia ior...");
        REPOS_IOR_FILE = iorUtils.getNewRefIOR("ClienteCorba");
        PrintUtil.Debug("\nNombre obtenido:" + REPOS_IOR_FILE);

        PrintUtil.Debug("\nLoad nueva referencia a Core Package...");
        pkgCore  = (_CorePackage)iorUtils.loadNewRefPkg("_CorePackage");
        PrintUtil.Debug("\nLoading referencia a Transformation Package...");
        pkg = (_CWMTransformationPackage)iorUtils.loadNewRefPkg("_CWMTransformationPackage");

        PrintUtil.Debug("\nCreando Transformation Package ...");
        crearTransformation(pkg,pkgCore);
        PrintUtil.PrintPackageTransformacional(pkg);

        PrintUtil.Debug("\nStoring referencia a Transformation Package en " +
        REPOS_IOR_FILE + "...");
        iorUtils.storeRef(REPOS_IOR_FILE,iorUtils.getORB().object_to_string(pkg));
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

  }


  /**
   * Dada la referencia a un package transformation y a uno core, crea un
   * paquete transformation en particular.
   * @param pkg referencia a un package Transformation
   * @param pkgCore referencia a un package Core
   * */
  private static void crearTransformation(_CWMTransformationPackage pkg,_CorePackage pkgCore)
    throws Exception{
      ManejoPackage mp = new ManejoPackage();

      /*Se crean los dataObjects*/
      DataObjectSet dataObjectSet1  =
        mp.crearDataObject("Data_1",Core.VisibilityKind.vk_public,pkg);

      DataObjectSet dataObjectSet2  =
        mp.crearDataObject("Data_2",Core.VisibilityKind.vk_public,pkg);

      DataObjectSet dataObjectSet3  =
        mp.crearDataObject("Data_3",Core.VisibilityKind.vk_public,pkg);

      String [] leng = {"sql"};
      ProcedureExpression pe =
        pkgCore.procedure_expression_ref().create_procedure_expression(
                  "SELECT * FROM TradesSource",leng);

      /*Se crea la transformacion*/
      Transformation t = mp.crearTransformacion("TRDExtractor",
                              Core.VisibilityKind.vk_public,
                              pe,"Consulta de la tabla de trades",true,pkg);
      /*Se asocian el fuente y destino a la transformacion*/
      t.add_source(dataObjectSet1);
      t.add_target(dataObjectSet2);

      /*Se crea un transformation task*/
      TransformationTask task = pkg.transformation_task_ref().create_transformation_task(
        "Tarea1",Core.VisibilityKind.vk_public,false);
      /*Se asocian esta tarea el transformation task creado anteriormente*/
      task.add_transformation(t);

      /*Se crea un paso*/
      TransformationStep step =
                  pkg.transformation_step_ref().create_transformation_step(
                  "Paso_1",Core.VisibilityKind.vk_public);

      /*Se asocian este paso a la tarea creada anteriormente*/
      step.set_task(task);

      /*Se crea la Actividad*/
      TransformationActivity actividad =
                  pkg.transformation_activity_ref().create_transformation_activity(
                  "Actividad_1",Core.VisibilityKind.vk_public,false,
                  new Timestamp(System.currentTimeMillis()).toString());
      ModelElement [] elems = {(ModelElement)step};
      actividad.set_step(elems);

      ProcedureExpression pe2 =
        pkgCore.procedure_expression_ref().create_procedure_expression(
                  "SELECT * FROM TradesSource WHERE Sales < 4",leng);
      /*Se crea otra transformacion*/
      Transformation t2 = pkg.transformation_ref().create_transformation(
                              "DEC97Trades",
                              Core.VisibilityKind.vk_public,
                              pe2,"Filtro de la tabla de trades",false);
      /*Se asocian el fuente y destino a la transformacion*/
      t2.add_source(dataObjectSet2);
      t2.add_target(dataObjectSet3);
      task.add_transformation(t2);

      /*Se crea otro paso*/
      TransformationStep step2 =
                  pkg.transformation_step_ref().create_transformation_step(
                  "Paso_2",Core.VisibilityKind.vk_public);
      /*Se asocian este paso a la tarea creada anteriormente*/
      step.set_task(task);

      String [] kind = {"Usage"};
      StepPrecedence stepPrec =
        pkg.step_precedence_ref().create_step_precedence("precedencia1",
          Core.VisibilityKind.vk_public, kind);

      //Elems tiene solo al paso step
      stepPrec.set_preceding_step(elems);

      ModelElement [] elems2 = {step2};
      stepPrec.set_succeeding_step(elems2);
  }

  private static String getAccion(String [] args){
    if (args==null || args.length==0){
      PrintUtil.Debug("Ingrese una opcion:" +
                          "\n\t--new"+
                          "\n\t--list" +
                          "\n\t--addnew");
      return null;
    }
    else if ((!args[0].equals("--new"))&&(!args[0].equals("--list"))&&(!args[0].equals("--addnew"))){
      PrintUtil.Debug("Opcion no valida");
      return null;
    }
    else
      return args[0];

  }




}